package nl.erdf.optimizer;

import nl.erdf.datalayer.DataLayer;
import nl.erdf.datalayer.QueryPattern;
import nl.erdf.model.Request;
import nl.erdf.model.Solution;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Node_Variable;

/**
 * @author tolgam
 * 
 */
public class Provider {
	/** Logger */
	final static Logger logger = LoggerFactory.getLogger(Provider.class);

	/** The data layer to get data from */
	final DataLayer datalayer;

	/** Subject,predicate and object */
	private final Node s;
	private final Node p;
	private final Node o;

	/**
	 * @param datalayer
	 * @param s
	 * @param p
	 * @param o
	 */
	public Provider(DataLayer datalayer, Node s, Node p, Node o) {
		this.datalayer = datalayer;
		this.s = s;
		this.p = p;
		this.o = o;
	}

	/**
	 * @param variable
	 * @param solution
	 * @return the query generated by this provider
	 */
	public QueryPattern getQuery(Node_Variable variable, Solution solution) {
		// Instantiate the pattern into a full triple
		Node subject = s;
		if (s.isVariable())
			subject = solution.getBinding((Node_Variable) s).getValue();

		Node predicate = p;
		if (p.isVariable())
			predicate = solution.getBinding((Node_Variable) p).getValue();

		Node object = o;
		if (o.isVariable())
			object = solution.getBinding((Node_Variable) o).getValue();

		// Replace the target variable by a return and create a query
		QueryPattern query = null;
		if ((s.isVariable()) && (variable.equals(s)))
			query = new QueryPattern(QueryPattern.RETURN, predicate, object);
		if ((p.isVariable()) && (variable.equals(p)))
			query = new QueryPattern(subject, QueryPattern.RETURN, object);
		if ((o.isVariable()) && (variable.equals(o)))
			query = new QueryPattern(subject, predicate, QueryPattern.RETURN);

		return query;
	}

	/**
	 * @param request
	 * @param variable
	 * @param solution
	 * @return the expected reward
	 */
	public double getExpectedReward(Request request, Node_Variable variable, Solution solution) {
		double reward = 0;

		if (s.isVariable())
			reward += solution.getBinding((Node_Variable) s).getReward() / request.getMaximumReward((Node_Variable) s);
		else
			reward += 1;

		if (p.isVariable())
			reward += solution.getBinding((Node_Variable) p).getReward() / request.getMaximumReward((Node_Variable) p);
		else
			reward += 1;

		if (o.isVariable())
			reward += solution.getBinding((Node_Variable) o).getReward() / request.getMaximumReward((Node_Variable) o);
		else
			reward += 1;

		return reward / 3;
	}
}
